/*
    Sega Genesis / Mega Drive startup file
    copyright (c) 2004,2005, Utkan Gngrd (fdarkangel@freeconsole.org)

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
*/

   
/* when defined, console name will be "SEGA GENESIS    ".
   else, console name will be "SEGA MEGA DRIVE " */

#define GENESIS

/* when defined, sram is enabled */
#define HAVE_SRAM

/* 1 if backup, 0 if not backup */
#define SRAM_BACKUP     1

/* 2 for even-address-only, 3 for odd-addresses-only, 0 for both */
#define SRAM_ADR_MODE   0



/* when defined, modem is enabled */
/* #define HAVE_MODEM */

/* modem firm name, 4 bytes ascii */
#define MODEM_FIRM "FIRM"

/* modem no, 2 bytes ascii */
#define MODEM_NO "12"

/* modem version, 1 byte ascii */
#define MODEM_VERSION "1"


/* supported devices, 12 bytes ascii. legend is as follows
J = joypad, 4 = team play, 6 = 6-button joypad, 0 = joystick for master system,
K = keyboard, R = serial rs232c, P = printer,T = tablet, B = control ball,
V = paddle controller, F = floppy disk drive, C = cd-rom, L = activator, M = mega mouse */

#define CONTROLLERS "JD              "

/* #define CLEAR_WRAM */

/* #define CLEAR_REGS */

/* #define ENABLE_INTERRUPTS */

/* if you want an external function called when an interrupt
   occurs, define the following. when defined, following global
   functions will be assumed to be defined elsewhere:
   
   void __ext_intr_handler()
   void __hblank_intr_handler()
   void __vblank_intr_handler()
*/
   
/* #define HAVE_EXTERN_INTERRUPT_HANDLERS */

.section .text

	| interrupt vectors table
	
	.long   __stack_start  | initial isp
	.long   _start         | initial pc

	| m68000 exception vectors

	.long   null_int   | access fault
	.long   null_int   | address error

	.long   null_int   | illegal instruction
	.long   div0_int   | division by 0
	.long   null_int   | chk, chk2 insturction
	.long   null_int   | ftrapcc, trapcc, trapv instructions

	.long   null_int   | privilage violation
	.long   null_int   | trace
	.long   null_int   | line a (1010) emulator
	.long   null_int   | line f (1111) emulator
	
	.long   null_int   | reserved
	.long   null_int   | co-processor protocol violation
	.long   null_int   | format error
	.long   null_int   | uninitialized interrupt
	
	.long   null_int   | reserved
	.long   null_int   | reserved
	.long   null_int   | reserved
	.long   null_int   | reserved
	.long   null_int   | reserved
	.long   null_int   | reserved
	.long   null_int   | reserved
	.long   null_int   | reserved

	.long   null_int   | spurious interrupt
	.long   null_int   | level 1 interrupt autovector
	.long   ext_int    | level 2 interrupt autovector
	.long   null_int   | level 3 interrupt autovector
	.long   hblank_int | level 4 interrupt autovector
	.long   null_int   | level 5 interrupt autovector
	.long   vblank_int | level 6 interrupt autovector
	.long   null_int   | level 7 interrupt autovector
	
	| trap #0 d 15 instruction vectors
	.long   null_int   | trap 0x0
	.long   null_int   | trap 0x1
	.long   null_int   | trap 0x2
	.long   null_int   | trap 0x3
	.long   null_int   | trap 0x4
	.long   null_int   | trap 0x5
	.long   null_int   | trap 0x6
	.long   null_int   | trap 0x7
	.long   null_int   | trap 0x8
	.long   null_int   | trap 0x9
	.long   null_int   | trap 0xa
	.long   null_int   | trap 0xb
	.long   null_int   | trap 0xc
	.long   null_int   | trap 0xd
	.long   null_int   | trap 0xe
	.long   null_int   | trap 0xf

	| floating point exceptions
	.long	null_int, null_int, null_int, null_int
	.long	null_int, null_int, null_int, null_int
	
	.long   null_int   | mmu config error
	.long   null_int   | mmu illegal operation error
	.long   null_int   | mmu access level operation error
	
	| reserved
	.long   null_int
	.long	null_int, null_int, null_int, null_int
#if 0
	| user defined vectors
	.long	null_int, null_int, null_int, null_int
	.long	null_int, null_int, null_int, null_int
	.long	null_int, null_int, null_int, null_int
	.long	null_int, null_int, null_int, null_int
#endif
	
	| ROM header

	| console name
#ifdef GENESIS
	.ascii "SEGA GENESIS    "
#else
	.ascii "SEGA MEGA DRIVE "
#endif
	
	| copyright info
	.ascii "(C)SEGA 2005.JUN"

	| domestic name
	.ascii "                                                "

	| overseas name
	.ascii "                                                "

	| GM (game), product code and serial
	.ascii "GM 12345678-01"
               
	| checksum will be here
	.word 0x0000

	| which devices are supported ?
	.ascii CONTROLLERS

	| ROM start address
	.long __rom_start
	| ROM end address
	.long __rom_end

	| RAM start address
	.long __ram_start
	| RAM end address
	.long __ram_end

	| external/save RAM info
#ifdef HAVE_SRAM
        .ascii "RA"
        .byte  0xa0 | (SRAM_BACKUP<<6) | (SRAM_ADR_MODE<<3)
        .ascii " "
        .long __sram_start
        .long __sram_end
#else
	.ascii "            "
#endif

#ifdef HAVE_MODEM
	.ascii "MO" MODEM_FIRM MODEM_NO "." MODEM_VERSION
#else
	.ascii "            "
#endif
	| memo (?)
	.ascii "                                        "
	| Country List
	.ascii "JUE             "

.global _start
_start:

#ifdef ENABLE_INTERRUPTS
	move.w  #0x2700, %sr
#else
	move.w  #0x2000, %sr
#endif

	tst.l   0xa10008
	bne     0f
	tst.w   0xa1000c
0:	bne	skip_setup


	clr.l   %d0
	move.b  0xa10001, %d0          | genesis / mega drive version reg
	beq     1f
	move.l  #0x53454741, 0xa14000  | ascii code for "SEGA"

1:	move.l  #0, %sp
	move.l  %sp, %a0

#ifdef CLEAR_WRAM
	| clear wram
	clr.l   %d0
	movea.l %sp, %a0
	move.l  #(__wram_size), %d1    | assumed to be defined in lnkscript
	subq.l  #1, %d1
2:	move.l  %d0, -(%a0)
	dbra    2b
#endif

#ifdef CLEAR_REGS
	movem.l %d0, %d1-%d7/%a0-%a6
#endif


/*
sr layout: (info from official m68000 manual)

bit(s)  |
---------
0         carry
1         overflow
2         zero
3         negative
4         extend
5-7       unused (always 0?)
8-10      interrupt priority mask
11        unused (always 0?)
12        (M) master/interrupt state
13        (S) supervisor/user state
14-15     (T0-T1) trace enable


S  |M  |Active Stack
0    x    usp
1    0    isp
1    1    msp


T0  | T1  |
0     0     no trace
1     0     trace on any instruction
0     1     trace on change of flow
1     1     undefined

*/


	
skip_setup:
	jmp     main


|==============================================================================
| interrput routines
|==============================================================================

#ifdef ENABLE_INTERRUPTS
div0_int:
#ifdef HAVE_LIBGEN
	pea       div0_str
	jbsr 	  gen_puts
	addq.l    #4, %sp
#endif
	rte

ext_int:
#ifdef HAVE_EXTERN_INTERRUPT_HANDLERS
	jbsr	__ext_intr_handler
#else
	/* write your ext intr handler code here */
#endif /* HAVE_EXTERN_INTERRUPT_HANDLERS */
	rte
	
hblank_int:
#ifdef HAVE_EXTERN_INTERRUPT_HANDLERS
	jbsr	__hblank_intr_handler
#else
	/* write your ext intr handler code here */
#endif /* HAVE_EXTERN_INTERRUPT_HANDLERS */
	rte

vblank_int:
#ifdef HAVE_EXTERN_INTERRUPT_HANDLERS
	jbsr	__vblank_intr_handler
#else
	/* write your ext intr handler code here */
#endif /* HAVE_EXTERN_INTERRUPT_HANDLERS */
	rte

#else /* ENABLE_INTERRUPTS */

div0_int:
ext_int:
hblank_int:
vblank_int:

#endif /* ENABLE_INTERRUPTS */

null_int:
	rte



#ifdef ENABLE_INTERRUPTS

.section .rodata

#ifdef HAVE_LIBGEN

div0_str: .string "division by zero\n"

#endif /* HAVE_LIBGEN */

#endif /* ENABLE_INTERRUPTS */

.end
